/******************************************************************************/ 
/* File Name:   I2C_LIB_KOE.c                                                 */
/* Autor: 		Michael Koefinger                                             */
/* Version: 	V1.00                                                         */
/* Date: 		16/12/2013                                                    */
/* Description: I2C_Library fr STM32F10x				               		  */
/******************************************************************************/
/* History: 	V1.00  creation										          */
/******************************************************************************/
#include "I2C_LIB_KOE.h"
/********************************************************************************************************************************/
/* \ *\	   U N T E R P R O G R A M M:		I2C_error_message(char errorcode)    										   \* / */
/* / *\--------------------------------------------------------------------------------------------------------------------\* \ */
/* \ *\	  Funktion:			 Gibt eine Fehleranalyse am Uart (1) aus. Der Fehler wird weitergegeben um das UPro	direkt in  \* / */
/* / *\						 einer Fehlerabfrage verwenden zu können:  "if(I2C_error_message(I2C1_init())!=info_success)"  \* \ */
/* \ *\	  Fehlerposition:	 -								   															   \* / */
/* / *\	  Übergabeparameter: Fehlercode																					   \* \ */
/* \ *\	  Rückgabewert:	     Fehlercode								   													   \* / */
/* / *\	  Nächster Schritt:  -																							   \* \ */
/********************************************************************************************************************************/
char I2C_error_message(char errorcode)
{
	if(errorcode==info_success)						//Kein Fehler
	{	return info_success;
	}
	uart_put_string("\n\r=== I2C-Error ===\n\r");
	uart_put_string(    "*****************\n\r\n\r");
	uart_put_string(    "   Fehlerposition:   		<");
	uart_put_hex(errorcode&0xF0);
	uart_put_string("> hex\n\r");
	uart_put_string(    "   Fehlerart:        		<");
	uart_put_hex(errorcode&0x0F);
	uart_put_string("> hex\n\r");
    uart_put_string(    "Analyse:\n\r");
	uart_put_string(    "========\n\r");
	uart_put_string(    "   Der Fehler ist ");
	switch(errorcode&0xF0)
	{	case err_at_init:  		uart_put_string("in der Initialisierung");							break;
		case err_at_start:		uart_put_string("bei der Startbedingung oder der Adressierung");	break;
		case err_at_send:		uart_put_string("beim Senden");										break;
		case err_at_read:		uart_put_string("beim Lesen");										break;
	}
	uart_put_string(    " aufgetreten.\n\r");
	uart_put_string(    "   Fehler-Beschreibung:  	\"");
	switch(errorcode&0x0F)
	{	case error_I2C1_running:		uart_put_string("error_I2C1_running");						break;
		case error_I2C1_disabled:		uart_put_string("error_I2C1_disabled");						break;
		case error_no_master:			uart_put_string("error_no_master");							break;
		case error_no_transmitter:		uart_put_string("error_no_transmitter");					break;
		case error_not_addressed:		uart_put_string("error_not_addressed");						break;
		case error_timeout_busy:		uart_put_string("error_timeout_busy");						break;
		case error_timeout_start:		uart_put_string("error_timeout_start");						break;
		case error_timeout_stop:		uart_put_string("error_timeout_stop");						break;
		case error_timeout_send:		uart_put_string("error_timeout_send");						break;
		case error_NACK:				uart_put_string("error_NACK");								break;
		case error_no_data:				uart_put_string("error_no_data");							break;
		case error_no_receiver: 		uart_put_string("error_no_receiver");						break;
	}
	uart_put_string(    "\".\n\r\n\r");
	return errorcode;
}

/******************************************************************************/
/*            U N T E R P R O G R A M M:    init_i2c_master              	  */
/*                                                                            */
/* Aufgabe: initialisiert den I2C Bus als Master Transmitter mit 100kHz Takt  */
/* Input:                                                                     */
/* return:	error message                                                     */
/******************************************************************************/
char I2C1_init(void) 
{
	int temp, err_cnt;

	RCC->APB1ENR |= 0x200000;				  // I2C Clock enable in APB1ENR
	RCC->APB1RSTR &= ~0x00200000;					// Peripheral Reset Register (I2C1RST=0)
	AFIO->MAPR &=  0xFFFFFFFD;					// I2C1_REMAP - Flag  (Remap auf 0 setzen --> PB6=SCL; PB7=SDA)

	RCC->APB2ENR |= RCC_APB2ENR_IOPBEN;       // enable clock for GPIOB	(APB2 Peripheral clock enable register)
	
	
	// PB7 & PB6 auf alt. function open drain für I2C
	temp = GPIOB->CRL; // Seite 105
	temp &= 0x00FFFFFF;	// PB7 & PB6 Konfigurationsbits löschen
	temp |= 0xFF000000; // PB7(SDA) & PB6(SCL) alt function open drain
	GPIOB->CRL = temp;
	err_cnt=0;
	while((I2C1->SR2 & BUSY) !=0)  //warten bis BUSY bit gelöscht ist
	{							  	// BUSY ist gesetzt wenn am I2C bus eine Übertragung läuft
		err_cnt++;
		if(err_cnt == 255)
		{
			return (err_at_init|error_timeout_busy);
		}
	}

	while((I2C1->CR1 & PE) !=0)	 //wenn PE gesetzt ist, dann ist der I2C bus
	{							 //bereits initialisiert
		return (err_at_init|error_I2C1_running);
	}

	temp = I2C1->CR2;
	temp &= 0xFFC0;
	temp |= 0x08; //für 8MHz		  
	I2C1->CR2 = temp;
	
	temp = I2C1->CCR;	//frequency for I2C CLK Pin
	temp &= 0x7000;
	temp |= 0x28;
	I2C1->CCR = temp;
	
	temp = I2C1->TRISE;
	temp &= 0xFFC0;
	temp |= 0x09;
	I2C1->TRISE = temp;
	
	temp = I2C1->CR1; //enable peripherals 
	temp &= 0xFFFE;
	temp |= 0x01;
	I2C1->CR1 = temp;
	
	return info_success;	
} 
/******************************************************************************/
/*            U N T E R P R O G R A M M:    I2C1_start                        */
/*                                                                            */
/* Aufgabe:   sendet Start, daten und stop bits				                  */
/* Input:     slave adresse        											  */
/* return:	 error                                                            */
/******************************************************************************/
char I2C1_start(char address, uint8_t read_anz)
{
 	int temp, err_cnt;

	while((I2C1->CR1 & PE) == 0)
	{
		return (err_at_start|error_I2C1_disabled);
	}

	temp = I2C1->CR1;	  //send Start bit
	temp &= 0xFEFF;
	temp |= 0x100;
	I2C1->CR1 = temp;
	
	err_cnt=0;
	while((I2C1->SR1 & SB) == 0)	 //warten bis start bit gesendet
	{
		err_cnt++;
		if(err_cnt==255)
		{
			return (err_at_start|error_timeout_start);
		}
	}
	
	while((I2C1->SR2 & MSL) == 0)	//warten ob Master Mode aktive ist 
	{
		return (err_at_start|error_no_master);
	}	

	//slave address senden 
	I2C1->DR = address;

	temp = I2C1->SR2;	//SR2 lesen 

	if((( address & 0x01 )==1) && (read_anz==1))
	{
		temp = I2C1->CR1;	 //ACK deaktiviern
		temp &= ~(0x400);
		temp |= 0x000;
		I2C1->CR1 = temp;
	}
	else if((( address & 0x01 )==1) && (read_anz==2))
	{
		temp = I2C1->CR1;	 //ACK & POS aktiviern
		temp &= ~(0xC00);
		temp |= 0xC00;
		I2C1->CR1 = temp;
	}
	else if(( address & 0x01 )==1)
	{
		temp = I2C1->CR1;	 //ACK aktiviern
		temp &= ~(0x400);
		temp |= ACK;
		I2C1->CR1 = temp;
	}

	err_cnt=0;
	while((I2C1->SR1 & ADDR) == 0) //warten bis slave address gesendet
	{
	 	err_cnt++;
		if(err_cnt==255)
		{
			return (err_at_start|error_not_addressed);
		}
	}

	return info_success;
}

/******************************************************************************/
/*            U N T E R P R O G R A M M:    I2C1_send                         */
/*                                                                            */
/* Aufgabe:   sendet daten und stop bit						                  */
/* Input:     daten, stop bit        										  */
/* return:	 error                                                            */
/******************************************************************************/
char I2C1_send(char *data, uint8_t anz)
{
	int temp, err_cnt, cnt;

	while((I2C1->SR2 & MSL) == 0)	   //Master modus aktiv??
	{
		return (err_at_send|error_no_master);
	}

	while((I2C1->SR2 & TRA) == 0)	   //master transmitter ??
	{
		return (err_at_send|error_no_transmitter);
	}

	for(cnt=1;cnt<=anz;cnt++)
	{
		I2C1->DR = data[cnt-1];

		err_cnt=0;
		while((I2C1->SR1 & BTF) == 0)	  //warten bis byte gesendet
		{
			err_cnt++;
			if((I2C1->SR1 & AF) != 0)
			{
				return (err_at_send|error_NACK);
			}
			if(err_cnt == 255)
			{
				return (err_at_send|error_timeout_send);
			}
		}
	}


	temp = I2C1->CR1;	//stop bit senden
	temp &= 0xFDFF;
	temp |= 0x200;
	I2C1->CR1 = temp;
	err_cnt=0;
	while((I2C1->SR2 & MSL) != 0)
	{
		err_cnt++;
		if(err_cnt == 255)
		{
			return (err_at_send|error_timeout_stop);
		} 	
	}
		
	return info_success;	
}	

/******************************************************************************/
/*            U N T E R P R O G R A M M:    I2C1_read                         */
/*                                                                            */
/* Aufgabe:   reads data from slave device						                  */
/* Input:     daten, stop bit        										  */
/* return:	 error                                                            */
/******************************************************************************/
char I2C1_read(char *data)
{
	int temp, err_cnt;

	while((I2C1->SR2 & MSL) == 0)		//master?
	{
		return (err_at_read|error_no_master);
	}

	while((I2C1->SR2 & TRA) == 1)	   //master receiver ??
	{
		return (err_at_read|error_no_receiver);
	}

	while((I2C1->SR1 & RxNE) == 0)	//ob daten empfangen
	{
		return (err_at_read|error_no_data);
	}

	temp = I2C1->CR1;	//stop bit senden
	temp &= 0xFDFF;
	temp |= 0x200;
	I2C1->CR1 = temp;
	err_cnt=0;
	while((I2C1->SR2 & MSL) != 0)
	{
		err_cnt++;
		if(err_cnt == 255)
		{
			return (err_at_send|error_timeout_stop);
		} 	
	}

	*data = I2C1->DR;

   	return info_success;
}

/******************************************************************************/
/*            U N T E R P R O G R A M M:    I2C1_read_n                       */
/*                                                                            */
/* Aufgabe:   reads data from slave device						              */
/* Input:     daten, stop bit        										  */
/* return:	 error                                                            */
/******************************************************************************/
char I2C1_read_n(char *data, uint8_t anz)
{
	int temp, err_cnt;

	while((I2C1->SR2 & MSL) == 0)		//master?
	{
		return (err_at_read|error_no_master);
	}

	while((I2C1->SR2 & TRA) == 1)	   //master receiver ??
	{
		return (err_at_read|error_no_receiver);
	}

	switch(anz)
	{
		case 2:
				temp = I2C1->CR1;	 //ACK deaktiviern
				temp &= ~(0x400);
				temp |= 0x000;
				I2C1->CR1 = temp;

				err_cnt=0;
				while((I2C1->SR1 & BTF) == 0)	  //warten bis byte empfangen
				{
					err_cnt=0;
					if((I2C1->SR1 & AF) != 0)
					{
						return (err_at_send|error_NACK);
					}
					if(err_cnt == 255)
					{
						return (err_at_send|error_timeout_send);
					}
				}

				temp = I2C1->CR1;	//stop bit senden
				temp &= 0xFDFF;
				temp |= 0x200;
				I2C1->CR1 = temp;
				err_cnt=0;
				while((I2C1->SR2 & MSL) != 0)
				{
					err_cnt++;
					if(err_cnt == 255)
					{
						return (err_at_send|error_timeout_stop);
					} 	
				}
			
				data[0] = I2C1->DR;
				data[1] = I2C1->DR;
				break;
		case 3:	 
				while((I2C1->SR1 & RxNE) == 0)	//ob daten empfangen
				{
					return (err_at_read|error_no_data);
				}

				err_cnt=0;
				while((I2C1->SR1 & BTF) == 0)	  //warten bis byte empfangen
				{
					err_cnt++;
					if((I2C1->SR1 & AF) != 0)
					{
						return (err_at_send|error_NACK);
					}
					if(err_cnt == 255)
					{
						return (err_at_send|error_timeout_send);
					}
				}

				temp = I2C1->CR1;	 //ACK deaktiviern
				temp &= ~(0x400);
				temp |= 0x000;
				I2C1->CR1 = temp;

				data[0] = I2C1->DR;

				temp = I2C1->CR1;	//stop bit senden
				temp &= 0xFDFF;
				temp |= 0x200;
				I2C1->CR1 = temp;
				err_cnt=0;
				while((I2C1->SR2 & MSL) != 0)
				{
					err_cnt++;
					if(err_cnt == 255)
					{
						return (err_at_send|error_timeout_stop);
					} 	
				}

				data[1] = I2C1->DR;

				while((I2C1->SR1 & RxNE) == 0)	//ob daten empfangen
				{
					return (err_at_read|error_no_data);
				}

				data[2] = I2C1->DR;
				break;
	}

   	return info_success;
}
/******************************************************************************/
/*            U N T E R P R O G R A M M:    get_dac_addr                      */
/*                                                                            */
/* Aufgabe:   testet alle moeglichen I2C Addr					              */
/* Input:                          											  */
/* return:	 	                                                      */
/******************************************************************************/
void I2C1_get_addr(void)
{
	char addr=0x00, err_msg;

	lcd_init();
	uart_init(9600);
	lcd_clear();
	lcd_put_string("I2C SLAVE ADDR");	
	lcd_set_cursor(1,0);
			 
	for(addr=0x00;addr <= 0xFF; addr++)
	{
		err_msg=I2C1_start(addr,0);		
		if(err_msg == info_success)
		{
			lcd_put_string("0x");
			uart_put_string("0x");
			lcd_set_cursor(1,2);
			lcd_put_hex(addr);
			uart_put_hex(addr);
			lcd_put_string(" ");
		}
	}
}

