
/******************************************************************************/ 
/* File Name:   I2C_LIB_KOE.h                                                 */
/* Autor: 		Michael Köfinger                                              */
/* Version: 	V1.00                                                         */
/* Date: 		13/01/2014                                                    */
/* Description: Header File for I2C Lib              		  */
/******************************************************************************/
/* History: 	V1.00  creation										          */
/******************************************************************************/
#ifndef  I2C_LIB_KOE_H 		//keine Mehrfach definition
#define  I2C_LIB_KOE_H
/*_________________________________LIBRARIES__________________________________*/
#include <stm32f10x.h>
#include "std_stm32_f103rb.h"

/*___________________________________DEFINES__________________________________*/
#define DACADDR 0x82 //0. bit -> Writebit=0, 7.bit-1.bit -> slave address | Adresspins 0-2 = floating
#define GYROADDR 0xD1 
#define SB 0x01
#define BTF 0x04
#define ADDR 0x02
#define TRA 0x04
#define BUSY 0x02
#define MSL 0x01
#define PE 0x01
#define AF 0x400
#define ACK 0x400
#define RxNE 0x0040

//Fehlercodes: von Maier Lib
//************
//
// Ein Fehlercode beinhaltet zwei Arten von Daten: Wo der Fehler auftrat und was der Fehlergrund ist.
// Die ersten 4bit (MSB) geben an, in welcher Routine sich der Fehler ereignet hat (Fehlerposition), die
// letzten 4bit (LSB) geben die Fehlerart an. Ist kein Fehler aufgetreten, wird 0x00 als Code verwendet,
// die Fehlerposition nicht miteinberechnet.
// Fehlercode-Berechnung:  "Fehlercode" = "Fehlerposition" | "Fehlerart";
//
#define info_success         0x00	//Kein Fehler aufgetreten (immer 0x00)
/*Fehlerposition:*/
#define err_at_init			 0x80 	//Fehler in der Initialisierungsroutine
#define err_at_start   	 	 0x40   //Fehler in der Start- und Adressierungsroutine
#define err_at_send			 0x20	//Fehler in der Senderoutine
#define err_at_read			 0x10	//Fehler in der Empfangsroutine
/*Fehlerarten:*/
#define error_I2C1_running   0x01	//Der I2C1-Bus ist bereits in Betrieb und kann nicht mehr initialisiert werden
#define error_I2C1_disabled  0x02	//Es kann keine Startbedingung erzeugt werden, da der Bus außer Betrieb ist (Keine Initialisierung)
#define error_no_master      0x03	//Der I2C1-Bus befindet sich nicht im Mastermode ("start()": Peripherie-Fehler;  "send()": Startbedingung erzeugt?)
#define error_no_transmitter 0x04   //Es können keine Daten gesandt werden, da der Slavebaustein zum lesen Adressiert wurde (R/W-bit muss 0 sein)
#define error_not_addressed  0x05   //Die Hardware wurde nicht adressiert, NACK erhalten (Wurde die richtige Adresse eingestellt?)
#define error_timeout_busy   0x06	//Der Bus kann derzeit noch nicht initialisiert werden, da bereits ein Datenverkehr ausgeführt wird (Leitung auf "LOW" - PullUp-Widerstände vorhanden?)
#define error_timeout_start  0x07   //Bei der Startbedingung ist es zu einem Time-Out gekommen (Ist der Bus belegt (Multimaster-Mode)?, Ist vielleicht noch eine Datenübertragung im gange?)
#define error_timeout_stop	 0x08	//Bei der Stopbedingung ist es zu einem Time-Out gekommen (Ist vielleicht noch eine Datenübertragung im gange?)
#define error_timeout_send   0x09	//Die Daten wurden nicht übertragen (Timeout) (Verwendet der Slave Clock-Streching? Sind die Busleitungen auf "LOW" gefallen?)
#define error_NACK			 0x0A	//Der Slave hat auf die Daten mit einem NACK geantwortet (Der Fehler liegt auf der Slave-Seite oder es wurde dessen Übertragungsprotokoll verletzt/beendet)
#define error_no_data		 0x0B	//Bei "read()". Entweder wurde noch kein Baustein adresiert oder nach Beendigung der letzten Übertragung sind keine ausstehenden Daten mehr vorhanden (Adressiert?  "I2C1_data_available()" verwendet?)
#define error_no_receiver	 0x0C

#ifdef I2C_LIB_KOE_MOD
#define EXPORT
#else
#define EXPORT extern
#endif
/*___________________________ FUNCTION PROTOTYPES______________________________*/
EXPORT char I2C_error_message(char errorcode); 			//von Maier Lib
EXPORT char I2C1_init(void);
EXPORT char I2C1_start(char address, uint8_t read_anz);
EXPORT char I2C1_send(char *data, uint8_t anz);	  //char *data: array mit daten | int anz: Anzahl der bergebenen Bytes
EXPORT char I2C1_read(char *data);
EXPORT char I2C1_read_n(char *data, uint8_t anz);
EXPORT void I2C1_get_addr(void);  //I2C Addr von Slave wird auf LCD angezeigt

#undef  EXPORT
#endif /* __I2C_LIB_KOE_H */
