/******************************************************************************/
/*   (C) Copyright HTL - HOLLABRUNN  2009-2010 All rights reserved. AUSTRIA   */ 
/*                                                                            */ 
/* File Name:   std32_f103rb_std.c                                            */
/* Autor: 		Josef Reisinger                                               */
/* Version: 	V1.00                                                         */
/* Date: 		07/05/2010                                                    */
/* Description: Standard Library fr ARM Corttex M3                           */
/******************************************************************************/
/* History: V1.00  creation										              */
/* 			V1.01  09.06.2010 Add Bit banding Adresses for LED/Switsches Board*/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __STM32F103RB_STD_H
#define __STM32F103RB_STD_H

/* Includes ------------------------------------------------------------------*/
#include <stm32f10x.h>
#include <stdio.h>

/*----------------------------- Define Pins for LED and Swistches--------------*/
/* Einzel-Portpins im Bitbandbereich festlegen                                 */

#define GPIOA_IDR GPIOA_BASE + 2*sizeof(uint32_t)    // Calc peripheral address GPIOA IDR
#define GPIOA_ODR GPIOA_BASE + 3*sizeof(uint32_t)
#define GPIOB_IDR GPIOB_BASE + 2*sizeof(uint32_t)
#define GPIOB_ODR GPIOB_BASE + 3*sizeof(uint32_t)	 // Calc peripheral address GPIOB ODR
#define GPIOC_IDR GPIOC_BASE + 2*sizeof(uint32_t)
#define GPIOC_ODR GPIOC_BASE + 3*sizeof(uint32_t)

// Calc Bit Band Adress from peripheral address: a = peripheral address b = Bit number
#define BITBAND_PERI(a,b) ((PERIPH_BB_BASE + (a-PERIPH_BASE)*32 + (b*4)))  

#define pSW0  (volatile unsigned long *)(BITBAND_PERI(GPIOA_IDR,0))   // PA0
#define pSW1  (volatile unsigned long *)(BITBAND_PERI(GPIOA_IDR,1))   // PA1
#define pSW2  (volatile unsigned long *)(BITBAND_PERI(GPIOA_IDR,2))   // PA2
#define pSW3  (volatile unsigned long *)(BITBAND_PERI(GPIOA_IDR,3))   // PA3
#define pSW4  (volatile unsigned long *)(BITBAND_PERI(GPIOA_IDR,4))   // PA4
#define pSW5  (volatile unsigned long *)(BITBAND_PERI(GPIOA_IDR,5))   // PA5
#define pSW6  (volatile unsigned long *)(BITBAND_PERI(GPIOA_IDR,6))   // PA6
#define pSW7  (volatile unsigned long *)(BITBAND_PERI(GPIOA_IDR,7))   // PA7

#define pLED0  (volatile unsigned long *)(BITBAND_PERI(GPIOB_ODR,8))   // PB8
#define pLED1  (volatile unsigned long *)(BITBAND_PERI(GPIOB_ODR,9))   // PB9
#define pLED2  (volatile unsigned long *)(BITBAND_PERI(GPIOB_ODR,10))  // PB10
#define pLED3  (volatile unsigned long *)(BITBAND_PERI(GPIOB_ODR,11))  // PB11
#define pLED4  (volatile unsigned long *)(BITBAND_PERI(GPIOB_ODR,12))  // PB12
#define pLED5  (volatile unsigned long *)(BITBAND_PERI(GPIOB_ODR,13))  // PB13
#define pLED6  (volatile unsigned long *)(BITBAND_PERI(GPIOB_ODR,14))  // PB14
#define pLED7  (volatile unsigned long *)(BITBAND_PERI(GPIOB_ODR,15))  // PB15

/*---------------------------------- Definitions -----------------------------*/
#define LineLen     16                  /* Width of LCD (in characters)       */
#define NumLines     2                  /* Hight of LCD (in lines)            */


#ifdef STM32F103RB_STD_MOD
#define EXPORT
#else
#define EXPORT extern
#endif 

/* Exported functions ------------------------------------------------------- */
EXPORT void init_leds_switches(void); 
EXPORT void set_leds (char value);
EXPORT char get_actual_switches(void); 

EXPORT void wait_ms(int ms);
EXPORT void wait_10us(void);

EXPORT void nib2asc(char *nib);
EXPORT void asc2nib(char *asc);
EXPORT int hex2bcd(char hex);

EXPORT void uart_init(unsigned long baudrate);
EXPORT char uart_get_char(void); 
EXPORT void uart_put_char(char ch);
EXPORT void uart_put_hex(char c); 
EXPORT void uart_put_string(char *string);
EXPORT void uart_clear(void);
EXPORT void uart_setpos(char x,char y);

EXPORT void lcd_init (void);
EXPORT void lcd_write_cmd (unsigned char c);
EXPORT void lcd_write_data (unsigned char c);
EXPORT void lcd_put_char (char c);
EXPORT void lcd_put_hex(char c); 
EXPORT void lcd_set_cursor (int line_nr, int column_nr);
EXPORT void lcd_clear (void);
EXPORT void lcd_put_string (char *string);
EXPORT void lcd_bargraph (int value, int size);
EXPORT void lcd_bargraphXY (int pos_y, int pos_x, int value);

EXPORT unsigned short int adc1_convert(unsigned char channel);

#undef  EXPORT
#endif /* __STM32F103RB_STD_H */

/******************* (C) HTL - HOLLABRUNN  2009-2010 *****END OF FILE****/
